package main

import (
	"encoding/json"
	pb "github.com/hyperledger/fabric/protos/peer"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"github.com/hyperledger/fabric/protos/peer"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/security/requests"
	"gitlab.smartblocklab.com/smartcontracts/systempreferenceCC/model"
)

func setFieldConstraints(userParameter *requests.Parameter, systemParameter *requests.Parameter, stub chaincode.StubInterfaceAdapter, timeGetter chaincode.TimeGetter) pb.Response {
	record := model.FieldConstraintsList{}
	if err := json.Unmarshal(systemParameter.Param, &record); err != nil {
		return shim.Error(err.Error())
	} else {
		if err = record.Save(stub); err != nil {
			return shim.Error(err.Error())
		} else {
			return shim.Success(nil)
		}
	}
}

func getFieldConstraints(userParameter *requests.Parameter, systemParameter *requests.Parameter, stub chaincode.StubInterfaceAdapter, timeGetter chaincode.TimeGetter) pb.Response {
	if raw, err := stub.GetState(model.FieldConstraintsKey); err == nil && raw == nil {
		return model.EmptyFieldConstraintsList().Marshal()
	} else if err != nil {
		return shim.Error(err.Error())
	} else {
		record := model.FieldConstraintsList{}
		if err = json.Unmarshal(raw, &record); err != nil {
			return shim.Error(err.Error())
		} else {
			return record.Marshal()
		}
	}
}

func setEnabledCryptoCurrencies(userParameter *requests.Parameter, systemParameter *requests.Parameter, stub chaincode.StubInterfaceAdapter, timeGetter chaincode.TimeGetter) peer.Response {
	record := model.EnabledCryptoCurrencyList{}
	if err := json.Unmarshal(systemParameter.Param, &record); err != nil {
		return shim.Error(err.Error())
	} else {
		if err = record.Save(stub); err != nil {
			return shim.Error(err.Error())
		} else {
			return shim.Success(nil)
		}
	}
}

func getEnabledCryptoCurrencies(userParameter *requests.Parameter, systemParameter *requests.Parameter, stub chaincode.StubInterfaceAdapter, timeGetter chaincode.TimeGetter) peer.Response {
	if raw, err := stub.GetState(model.EnabledCryptoCurrenciesKey); err == nil && raw == nil {
		return model.EmptyEnabledCryptoCurrencyList().Marshal()
	} else if err != nil {
		return shim.Error(err.Error())
	} else {
		record := model.EnabledCryptoCurrencyList{}
		if err = json.Unmarshal(raw, &record); err != nil {
			return shim.Error(err.Error())
		} else {
			return record.Marshal()
		}
	}
}

func setCryptoCurrencyConstraints(userParameter *requests.Parameter, systemParameter *requests.Parameter, stub chaincode.StubInterfaceAdapter, timeGetter chaincode.TimeGetter) peer.Response {
	record := model.CryptoCurrencyConstraintsList{}
	if err := json.Unmarshal(systemParameter.Param, &record); err != nil {
		return shim.Error(err.Error())
	} else {
		if err = record.Save(stub); err != nil {
			return shim.Error(err.Error())
		} else {
			return shim.Success(nil)
		}
	}
}

func getCryptoCurrencyConstraints(userParameter *requests.Parameter, systemParameter *requests.Parameter, stub chaincode.StubInterfaceAdapter, timeGetter chaincode.TimeGetter) peer.Response {
	if raw, err := stub.GetState(model.EnabledCryptoCurrenciesKey); err == nil && raw == nil {
		return model.EmptyCryptoCurrencyConstraintsList().Marshal()
	} else if err != nil {
		return shim.Error(err.Error())
	} else {
		record := model.EnabledCryptoCurrencyList{}
		if err = json.Unmarshal(raw, &record); err != nil {
			return shim.Error(err.Error())
		} else {
			return record.Marshal()
		}
	}
}

func NewChaincode() *chaincode.CommonChaincode {
	chaincode.ChaincodeName = "systempreferenceCC"

	chaincode.ChaincodeVar.Actions[model.SetFieldConstraints] = setFieldConstraints
	chaincode.ChaincodeVar.Actions[model.GetFieldConstraints] = getFieldConstraints
	chaincode.ChaincodeVar.Actions[model.SetEnabledCryptoCurrencies] = setEnabledCryptoCurrencies
	chaincode.ChaincodeVar.Actions[model.GetEnabledCryptoCurrencies] = getEnabledCryptoCurrencies
	chaincode.ChaincodeVar.Actions[model.SetCryptoCurrencyConstraints] = setCryptoCurrencyConstraints
	chaincode.ChaincodeVar.Actions[model.GetCryptoCurrencyConstraints] = getCryptoCurrencyConstraints

	return &chaincode.ChaincodeVar
}
