package couchdb

import (
	"encoding/json"
	"github.com/sirupsen/logrus"
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"errors"
)

var logger = logrus.WithFields(logrus.Fields{
	"module":  "common",
	"package": "couchdb",
})

func GetSelector(data interface{}) (string, error) {
	logger.Info("getSelector")
	query, err := json.Marshal(struct {
		Selector interface{} `json:"selector"`
	}{
		Selector: data,
	})

	stringQuery := string(query)
	logger.Infof("query: %s", stringQuery)
	return stringQuery, err
}

func FindOneByQuery(queryObject interface{}, stub chaincode.StubInterfaceAdapter) ([]byte, error) {
	logger.Info("findOneByQuery")
	query, err := GetSelector(queryObject)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to create query. Error: %s", err)
		logger.Error(errMsg)
		return nil, errors.New(errMsg)
	}

	logger.Info("query: ", query)

	it, err := stub.GetQueryResult(query)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to query %s. Error: %s", query, err)
		logger.Error(errMsg)
		return nil, errors.New(errMsg)
	}

	defer it.Close() // Do not forget to close iterator once get used
	for it.HasNext() {
		record, err := it.Next()
		if err != nil {
			errMsg := fmt.Sprintf("Failed to read record from query iterator for query %s. Error: %s", query, err)
			logger.Error(errMsg)
			return nil, errors.New(errMsg)
		}

		return record.Value, nil
	}

	return nil, errors.New("not found")
}