package function

import (
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/security/tools"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model"
)

// AccountVerifier verifies account validity
type CheckAndLoadAccount_Type func(userAccount *model.UserAccountInfo, adapter chaincode.StubInterfaceAdapter) (*model.UserAccountInfo, error)

func CheckAndLoadAccount_Function(userAccountInfo *model.UserAccountInfo, adapter chaincode.StubInterfaceAdapter) (*model.UserAccountInfo, error) {
	logMsg := fmt.Sprintf("checkAndLoadAccountFunction (UserAccountInfo: %s)", userAccountInfo.String())
	logger.Error(logMsg)

	// Invoke account check chaincode
	resp := tools.SignedInvokeChaincode(adapter, invoke.AccountChannel, invoke.AccountCC, invoke.CheckAndLoadAccountFunction, userAccountInfo)
	// We got error status
	if resp.Status != int32(shim.OK) {
		logger.Errorf("verification of user id %s failed, with following response %s", userAccountInfo.UserID, resp.Message)
		return nil, errors.New(resp.Message)
	}
	// No error, account verified
	var userAccount model.UserAccountInfo
	err := json.Unmarshal(resp.Payload, &userAccount)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize user account info, due to %s", err)
		logger.Error(errMsg)
		return nil, err
	}
	return &userAccount, nil
}
