package function

import (
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/security/tools"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/model"
	"errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
)

type GetCommission_Type func(partnerCode string, customerType model.CustomerType, commissionType model.CommissionType, stub chaincode.StubInterfaceAdapter) (float64, error)

// CommissionForTransfer compute commission based on transaction input parameters
func GetCommission_Function(partnerCode string, customerType model.CustomerType, commissionType model.CommissionType, stub chaincode.StubInterfaceAdapter) (float64, error) {
	msg := fmt.Sprintf("CommissionForTransfer (partnerCode %s, customerType %s, commissionType %s)", partnerCode, customerType, commissionType)
	logger.Error(msg)

	// проверяем partner code
	// 1. на существование
	// 2. на то что организация с этим партнер кодом не заблокированна
	organization, err := GetOrganization_Function(partnerCode, stub)
	if err != nil {
		errMsg := fmt.Sprintf("Catn't load organization by parnter code %s, due to %s", partnerCode, err.Error())
		logger.Error(errMsg)
		return 0, errors.New(errMsg)
	}

	if organization == nil {
		errMsg := fmt.Sprintf("Organization not found by parnter code %s", partnerCode)
		logger.Error(errMsg)
		return 0, errors.New(errMsg)
	}

	resp := tools.SignedInvokeChaincode(stub, invoke.CommissionChannel, invoke.CommissionCC, invoke.GetCommissionFunction, model.GetCommissionArg{
		PartnerCode:    partnerCode,
		CustomerType:   customerType.String(),
		CommissionType: commissionType.String(),
	})
	if resp.Status != int32(shim.OK) {
		errMsg := fmt.Sprintf("Error obtain commission for parnter code %s, customer type %s, commission type %s, due to %s", partnerCode, customerType.String(), commissionType.String(), resp.Message)
		logger.Error(errMsg)
		return 0, errors.New(errMsg)
	}

	var commission model.Commission
	err = json.Unmarshal(resp.Payload, &commission)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize commission, parnter code %s, due to %s", partnerCode, resp.Message)
		logger.Error(errMsg)
		return 0, errors.New(errMsg)
	}
	return commission.CommissionPercent, nil
}
