package function

import (
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/security/tools"
	"gitlab.smartblocklab.com/smartcontracts/common/security/requests"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"encoding/json"
	"errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
)

// CryptocurrencyLoader function to load information about available cryptocurrencies transfered within
// external crypto currencies wallets
type GetCryptocurrenciesByDaemonName_Type func(systemUserId string, adapter chaincode.StubInterfaceAdapter) (*model.CryptocurrencyStoreOutput, error)

func GetCryptocurrenciesByDaemonName_Function(systemUserId string, adapter chaincode.StubInterfaceAdapter) (*model.CryptocurrencyStoreOutput, error) {
	logMsg := fmt.Sprintf("cryptocurrencyLoaderFunction (daemonName: %s)", systemUserId)
	logger.Error(logMsg)

	resp := tools.SignedInvokeChaincodeWithUserId(adapter, systemUserId, invoke.GetCryptocurrenciesByDaemonNameChannel, invoke.CryptocurrencyCC, invoke.GetCryptocurrenciesByDaemonNameFunction, requests.Id{Id: systemUserId})

	// We got error status
	if resp.Status != int32(shim.OK) {
		logger.Errorf("Failed to invoke getCryptocurrenciesByDaemonName from cryptocurrencyCC, with following error message %s", resp.Message)
		return nil, errors.New(resp.Message)
	}
	// No error, account verified
	var cryptocurrencyStoreOutput model.CryptocurrencyStoreOutput
	err := json.Unmarshal(resp.Payload, &cryptocurrencyStoreOutput)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize user CryptocurrencyStoreOutput, due to %s", err)
		logger.Error(errMsg)
		return nil, err
	}
	return &cryptocurrencyStoreOutput, nil
}
