package function

import (
	"encoding/json"
	"errors"
	"fmt"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"gitlab.smartblocklab.com/smartcontracts/common/security/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/security/tools"
	"gitlab.smartblocklab.com/smartcontracts/common/model"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"github.com/sirupsen/logrus"
)

var logger = logrus.WithFields(logrus.Fields{
	"module":  "common",
	"package": "function",
})

type GetOrganization_Type func(partnerCode string, adapter chaincode.StubInterfaceAdapter) (*model.Organization, error)

func GetOrganization_Function(partnerCode string, adapter chaincode.StubInterfaceAdapter) (*model.Organization, error) {
	logger.Infof("organizationLoaderFunction (partnerCode: %s)", partnerCode)

	resp := tools.SignedInvokeChaincode(adapter, invoke.OrganizationChannel, invoke.OrganizationCC, invoke.GetOrganizationFunction, requests.Id{Id: partnerCode})

	if resp.Status == 404 {
		logger.Debug("Organization is not found")
		return nil, nil
	}

	// We got error status
	if resp.Status != int32(shim.OK) {
		logger.Errorf("Failed to invoke GetOrganization from organizationCC, with following error message %s", resp.Message)
		return nil, errors.New(resp.Message)
	}
	// No error, account verified
	var organization model.Organization
	err := json.Unmarshal(resp.Payload, &organization)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize user Organization, due to %s", err)
		logger.Error(errMsg)
		return nil, err
	}
	return &organization, nil
}
