package marketaccount

import (
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/interfaces/imarketaccount"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
	"gitlab.smartblocklab.com/smartcontracts/common/model/marketaccount"
	"gitlab.smartblocklab.com/smartcontracts/common/security/requests"
	"gitlab.smartblocklab.com/smartcontracts/common/security/tools"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
)

type RemoteManagerImpl struct {
	adapter    chaincode.StubInterfaceAdapter
	timeGetter chaincode.TimeGetter
	channel    string
}

func GetRemoteManager(adapter chaincode.StubInterfaceAdapter, timeGetter chaincode.TimeGetter, channel string) imarketaccount.IMarketaccountCC {
	return &RemoteManagerImpl{
		adapter:    adapter,
		timeGetter: timeGetter,
		channel:    channel,
	}
}

func (this *RemoteManagerImpl) AddWallet(UserID string, newWalletUISigned marketaccount.NewWallet, walletId requests.Id) (*marketaccount.Wallet, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this *RemoteManagerImpl) AddAccount(UserID string, newAccountUISigned marketaccount.NewAccount, accountId requests.Id) (*marketaccount.Account, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}

func (this *RemoteManagerImpl) AddExternalAccount(UserID string, newExternalAccount marketaccount.NewExternalAccount, subAccountId requests.Id) (*marketaccount.Account, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}

func (this *RemoteManagerImpl) ActivateWallet(UserID string, walletId requests.Id) (*marketaccount.Wallet, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this *RemoteManagerImpl) BlockWallet(UserID string, walletId requests.Id) (*marketaccount.Wallet, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this *RemoteManagerImpl) ActivateAccount(UserID string, accountId requests.Id) (*marketaccount.Account, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this *RemoteManagerImpl) BlockAccount(UserID string, accountId requests.Id) (*marketaccount.Account, error) {
	return nil, logging.Get().Error("Failed to make remote call")
}
func (this *RemoteManagerImpl) GetAccountByCurrencyType(arg marketaccount.GetAccountByCurrencyTypeArg) (*marketaccount.Account, error) {
	if resp := tools.SignedInvokeChaincode(this.adapter, this.channel, invoke.MarketaccountCC, invoke.GetAccountByCurrencyTypeFunction, arg); resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	} else {
		var result marketaccount.Account
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, logging.Get().Error(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *RemoteManagerImpl) VerifyUserAccount(arg marketaccount.AccountInfo) (*marketaccount.Account, error) {
	if resp := tools.SignedInvokeChaincode(this.adapter, this.channel, invoke.MarketaccountCC, invoke.VerifyUserAccountFunction, arg); resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	} else {
		var result marketaccount.Account
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, logging.Get().Error(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *RemoteManagerImpl) CheckAndLoadAccount(arg marketaccount.AccountInfo) (*marketaccount.Account, error) {
	if resp := tools.SignedInvokeChaincode(this.adapter, this.channel, invoke.MarketaccountCC, invoke.CheckAndLoadAccountFunction, arg); resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	} else {
		var result marketaccount.Account
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, logging.Get().Error(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *RemoteManagerImpl) GetAllUserWallets(userId requests.Id) (*marketaccount.WalletsEnvelope, error) {
	if resp := tools.SignedInvokeChaincode(this.adapter, this.channel, invoke.MarketaccountCC, invoke.GetAllUserWalletsFunction, userId); resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	} else {
		var result marketaccount.WalletsEnvelope
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, logging.Get().Error(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *RemoteManagerImpl) GetAccountInfoByAccountID(accountId requests.Id) (*marketaccount.AccountInfo, error) {
	if resp := tools.SignedInvokeChaincode(this.adapter, this.channel, invoke.MarketaccountCC, invoke.GetAccountInfoByAccountIDFunction, accountId); resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	} else {
		var result marketaccount.AccountInfo
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, logging.Get().Error(err.Error())
		} else {
			return &result, nil
		}
	}
}

func (this *RemoteManagerImpl) GetAccounts(userId requests.Id) (*marketaccount.AccountsEnvelope, error) {
	if resp := tools.SignedInvokeChaincode(this.adapter, this.channel, invoke.MarketaccountCC, invoke.GetAccountsFunction, userId); resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	} else {
		var result marketaccount.AccountsEnvelope
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, logging.Get().Error(err.Error())
		} else {
			return &result, nil
		}
	}
}
func (this *RemoteManagerImpl) GetWallets(userId requests.Id) (*marketaccount.WalletsEnvelope, error) {
	if resp := tools.SignedInvokeChaincode(this.adapter, this.channel, invoke.MarketaccountCC, invoke.GetWalletsFunction, userId); resp.Status != 200 {
		return nil, logging.Get().Error(resp.Message)
	} else {
		var result marketaccount.WalletsEnvelope
		if err := json.Unmarshal(resp.Payload, &result); err != nil {
			return nil, logging.Get().Error(err.Error())
		} else {
			return &result, nil
		}
	}
}