package model

import (
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/nums"
)

// CryptocurrencyStoreOutput captures output of crypto currency chaincode invocation
type CryptocurrencyStoreOutput struct {
	CryptocurrencyStore []CryptocurrencyStore `json:"cryptocurrencies"`
}

// CryptocurrencyStore encapsulates information about
// cryptocurrencies store
type CryptocurrencyStore struct {
	Time                  int64                      `json:"time"`
	ExternalTransactionId string                     `json:"external_transaction_id"`
	OutAccountInfo        *CryptocurrencyAccountInfo `json:"out_account_info_id"`
	InAccountInfo         *CryptocurrencyAccountInfo `json:"in_account_info_id"`
	Amount                nums.PlainCurrencyNumber                    `json:"amount"`
}

// CryptocurrencyAccountInfo keeps information about crypto currency
// account, including information about external wallet
type CryptocurrencyAccountInfo struct {
	UserID    string `json:"user_id"`
	WalletID  string `json:"wallet_id"`
	AccountID string `json:"account_id"`
}

// InKeyForCurrency generate key for "in" direction based on account information
// given currency id
func (info CryptocurrencyAccountInfo) InKeyForCurrency(currencyType string) string {
	return fmt.Sprintf("in.%s.%s", currencyType, info.AccountID)
}

// OutKeyForCurrency generate key for "out" direction based on account information
// given currency id
func (info CryptocurrencyAccountInfo) OutKeyForCurrency(currencyType string) string {
	return fmt.Sprintf("out.%s.%s", currencyType, info.AccountID)
}