package market

import "gitlab.smartblocklab.com/smartcontracts/common/nums"

type DocumentType string

type OperationType string

type OrderStatus string

type OrderType int

type MarketFundsArg struct {
	AccountID   string                   `json:"account_id"`
	OperationID string                   `json:"operation_id"`
	UserID      string                   `json:"user_id"`
	Amount      nums.PlainCurrencyNumber `json:"amount"`
}

type FindMarketTransactionArg struct {
	OperationID   string        `json:"operation_id"`
	UserID        string        `json:"user_id"`
	CurrencyType  string        `json:"currency_type"`
	AccountID     string        `json:"account_id"`
	OperationType OperationType `json:"operation_type"`
}

type MarketTransaction struct {
	Key           string                   `json:"key"`
	SessionID     int64                    `json:"session_id"`
	OperationID   string                   `json:"operation_id"`
	DocumentType  DocumentType             `json:"document_type"`
	OperationType OperationType            `json:"operation_type"`
	AccountID     string                   `json:"account_id"`
	Amount        nums.PlainCurrencyNumber `json:"amount"`
	Timestamp     int64                    `json:"timestamp"`
	DependencyKey string                   `json:"dependency_key"` // transaction dependency in a form of "txnType.currencyType.accountId"
}

type MarketTransactionList struct {
	MarketTransactions []MarketTransaction `json:"market_transactions"`
}

type MarketArg struct {
	UserID       string `json:"user_id"`
	CurrencyType string `json:"currency_type"`
	AccountID    string `json:"account_id"`
}

type UIMarketOrder struct {
	AccountID  string                   `json:"account_id"`
	Price      nums.PlainCurrencyNumber `json:"price"`
	Amount     nums.PlainCurrencyNumber `json:"amount"`
	StopPrice  nums.PlainCurrencyNumber `json:"stop_price"`
	LimitPrice nums.PlainCurrencyNumber `json:"limit_price"`
}

type UIDepositFunds struct {
	MarketAccountID string                   `json:"market_account_id"`
	AccountID       string                   `json:"account_id"`
	Amount          nums.PlainCurrencyNumber `json:"amount"`
}

type UIMarketWithdrawal struct {
	MarketAccountID string                   `json:"market_account_id"`
	AccountID       string                   `json:"account_id"`
	Amount          nums.PlainCurrencyNumber `json:"amount"`
}

type PlainMarketOrderList struct {
	PlainMarketOrders []PlainMarketOrder `json:"plain_market_orders"`
}

type PlainMarketOrder struct {
	UserID       string                   `json:"user_id"`
	OperationID  string                   `json:"operation_id"`
	SessionID    int64                    `json:"session_id"`
	AccountID    string                   `json:"account_id"`
	SellPrice    nums.PlainCurrencyNumber `json:"sell_price"`
	DocumentType DocumentType             `json:"document_type"`
	OrderStatus  OrderStatus              `json:"order_status"`
	BuyAmount    nums.PlainCurrencyNumber `json:"buy_amount"`
	FilledAmount nums.PlainCurrencyNumber `json:"filled_amount"`
	StopPrice    nums.PlainCurrencyNumber `json:"stop_price"`
	LimitPrice   nums.PlainCurrencyNumber `json:"limit_price"`
	Timestamp    int64                    `json:"timestamp"`
}

type MarketBalance struct {
	UserID    string                   `json:"user_id"`
	Balance   nums.PlainCurrencyNumber `json:"balance"`
	AccountID string                   `json:"account_id"`
}

type TransactionList struct {
	OrderTransactions  []OrderTransaction  `json:"order_transactions"`
	MarketTransactions []MarketTransaction `json:"market_transactions"`
}

/**
результат матчинга ордеров
*/
type OrderTransaction struct {
	Key           string                   `json:"key"`
	SequenceID    int                      `json:"sequence_id"`
	SessionID     int64                    `json:"session_id"`
	OutOrderID    string                   `json:"out_order_id"`
	InOrderID     string                   `json:"in_order_id"`
	Price         nums.PlainCurrencyNumber `json:"price"`
	Amount        nums.PlainCurrencyNumber `json:"amount"`
	DocumentType  DocumentType             `json:"document_type"`
	DependencyKey string                   `json:"dependency_key"`
}

type MarketOrder struct {
	SessionID      int64
	UserID         string
	OperationID    string
	StatusInternal OrderStatus // статус ордера: открыт/закрыт
	AccountID      string
	CurrencyPair   *CurrencyPair        // валютная пара
	OrderType      OrderType            // тип лимитного ордера: http://profitraders.com/Bases/Orders.html
	Price          *nums.CurrencyNumber // цена 1 единицы валюты, по какой цене хочу купить
	Amount         *nums.CurrencyNumber // количество единиц валюты, сколько хочу купить
	FilledAmount   *nums.CurrencyNumber // сколько валюты было обработано этим ордером
	Timestamp      int64                // дата создания ордера

	// Граница при продаже:
	// если текущая цена стала ниже StopPrice, то сразу же продаем
	StopPrice *nums.CurrencyNumber

	// Граница при продаже:
	// если текущая цена стала выше LimitPrice, то сразу же продаем
	LimitPrice *nums.CurrencyNumber
}

type MarketOrderList struct {
	MarketOrders []MarketOrder `json:"market_orders"`
}

// Описатель котировки
type CurrencyPair struct {
	BuyAsset  *nums.Asset `json:"buy_asset"`  // Что хотим купить
	SellAsset *nums.Asset `json:"sell_asset"` // Что хотим продать
}

// Опции лимитированного ордера
type Options struct {
	// Граница при продаже:
	// если текущая цена стала ниже StopPrice, то сразу же продаем
	StopPrice *nums.CurrencyNumber

	// Граница при продаже:
	// если текущая цена стала выше LimitPrice, то сразу же продаем
	LimitPrice *nums.CurrencyNumber
}
