package market

import (
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
)

func (obj *MarketTransaction) Save(adapter chaincode.StubInterfaceAdapter, outUserId, inUserId string, seed int) error {
	// Serialize transaction record into JSON
	jsonRecordLockFunds, err := json.Marshal(obj)
	if err != nil {
		return logging.Get().Errorf("failed to marhsal MarketTransaction, operation id %s, error %s", obj.RecordKey(outUserId, inUserId, seed), err)
	}

	logging.Get().Debugf("save MarketTransaction by key: %s", obj.RecordKey(outUserId, inUserId, seed))
	err = adapter.PutState(obj.RecordKey(outUserId, inUserId, seed), jsonRecordLockFunds)
	if err != nil {
		return logging.Get().Errorf("failed to add new MarketTransaction, operation id %s, error %s", obj.RecordKey(outUserId, inUserId, seed), err)
	}

	return nil
}

func (obj PlainMarketOrder) Save(adapter chaincode.StubInterfaceAdapter) error {
	// Serialize transaction record into JSON
	jsonRecordLockFunds, err := json.Marshal(obj)
	if err != nil {
		return logging.Get().Errorf("failed to marhsal PlainMarketOrder, operation id %s, error %s", obj.ObjectKey(), err)
	}

	logging.Get().Infof("save PlainMarketOrder by key: %s", obj.RecordKey())

	err = adapter.PutState(obj.RecordKey(), jsonRecordLockFunds)
	if err != nil {
		return logging.Get().Errorf("failed to add PlainMarketOrder for user id (%s), operation id %s, error %s", obj.UserID, obj.OperationID, err)
	}

	return nil
}

func (obj *OrderTransaction) Save(adapter chaincode.StubInterfaceAdapter) error {
	// Serialize transaction record into JSON
	jsonRecordLockFunds, err := json.Marshal(obj)
	if err != nil {
		return logging.Get().Errorf("failed to marhsal OrderTransaction, sequence_id=%d, session_id=%d, error %s", obj.SequenceID, obj.SessionID, err)
	}

	logging.Get().Debugf("save order transaction by key: %s", obj.RecordKey())
	err = adapter.PutState(obj.RecordKey(), jsonRecordLockFunds)
	if err != nil {
		return logging.Get().Errorf("failed to save OrderTransaction, recordKey=%s, error %s", obj.RecordKey(), err)
	}

	return nil
}
