package marketaccount

// DocType defines documents type to
// persist in state db
type DocType string

// Type used to identify wallet or account types
// possible values: MAIN and ADDITIONAL
type Type string

// Status encapsulates account/wallet
// statuses values
type Status string

// Wallet keeps representation of the
// user wallet
type Wallet struct {
	DocType  DocType `json:"doc_type,omitempty"`
	WalletID string  `json:"wallet_id,omitempty"`
	UserID   string  `json:"user_id,omitempty"`
	Type     Type    `json:"type,omitempty"`
	Status   Status  `json:"status,omitempty"`
}

// Account represents system account which includes
// information about account type and account wallets
type Account struct {
	DocType      DocType `json:"doc_type,omitempty"`
	AccountID    string  `json:"account_id,omitempty"`
	WalletID     string  `json:"wallet_id,omitempty"`
	UserID       string  `json:"user_id,omitempty"`
	Type         Type    `json:"type,omitempty"`
	Status       Status  `json:"status,omitempty"`
	CurrencyType string  `json:"currency_type,omitempty"`
}

// AccountInfo ids which uniquely identifies
// information about certain account which has to be validated
type AccountInfo struct {
	UserID    string `json:"user_id,omitempty"`
	WalletID  string `json:"wallet_id,omitempty"`
	AccountID string `json:"account_id,omitempty"`
	Status    Status `json:"status,omitempty"`
}

type NewWallet struct {
	Type Type `json:"type,omitempty"`
}
type NewAccount struct {
	WalletID string `json:"wallet_id,omitempty"`
	Type     Type   `json:"type,omitempty"`
}

// WalletsEnvelope struct to encapsulate list of
// all user wallets
type WalletsEnvelope struct {
	UserID  string                `json:"user_id,omitempty"`
	Wallets []*WalletWithAccounts `json:"wallet_list,omitempty"`
}

type WalletWithAccounts struct {
	Wallet   Wallet    `json:"wallet,omitempty"`
	Accounts []Account `json:"accounts,omitempty"`
}

type GetAccountByCurrencyTypeArg struct {
	UserID       string `json:"user_id,omitempty"`
	CurrencyType string `json:"currency_type,omitempty"`
}

type NewExternalAccount struct {
	WalletID     string `json:"wallet_id,omitempty"`
	CurrencyType string `json:"currency_type,omitempty"`
}

type AccountsEnvelope struct {
	Accounts []Account
}
