package marketaccount

import (
	"encoding/json"
	"fmt"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"github.com/hyperledger/fabric/protos/peer"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
)

func (obj *Account) Save(stub chaincode.StubInterfaceAdapter) peer.Response {
	objByteArray, err := json.Marshal(obj)
	if err != nil {
		errMsg := fmt.Sprintf("failed to Marshal. error %s", err)
		logging.Get().Error(errMsg)
		return shim.Error(errMsg)
	}

	err = stub.PutState(obj.AccountID, objByteArray)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to PutState. Error: %s", err)
		logging.Get().Error(errMsg)
		return shim.Error(errMsg)
	}

	return shim.Success(objByteArray)
}

func (obj *Wallet) Save(stub chaincode.StubInterfaceAdapter) peer.Response {
	objByteArray, err := json.Marshal(obj)
	if err != nil {
		errMsg := fmt.Sprintf("failed to Marshal. error %s", err)
		logging.Get().Error(errMsg)
		return shim.Error(errMsg)
	}

	err = stub.PutState(obj.WalletID, objByteArray)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to PutState. Error: %s", err)
		logging.Get().Error(errMsg)
		return shim.Error(errMsg)
	}

	return shim.Success(objByteArray)
}
