package model

// User struct which specifies user record
type User struct {
	UserID       string          `json:"user_id,omitempty"`
	PartnerCode  string          `json:"partner_code,omitempty"`
	CustomerType CustomerType    `json:"customer_type,omitempty"`
	UserStatus   UserStatus      `json:"user_status,omitempty"`
	PublicKeys   []UserPublicKey `json:"user_public_keys,omitempty"`
}

type UserStatus string

const (
	UserStatus_Moderation UserStatus = "MODERATION"
	UserStatus_Approved   UserStatus = "APPROVED"
	UserStatus_Rejected   UserStatus = "REJECTED"
	UserStatus_Blocked    UserStatus = "BLOCKED"
	UserStatus_Activated  UserStatus = "ACTIVATED"
)

// UserPublicKey represents user public key
// currently we support only X509 format
type UserPublicKey struct {
	Alias     string `json:"alias,omitempty"`
	PublicKey string `json:"publicKey,omitempty"`
	// We need to add signature, so we will be able to verify whenever user which publishes information
	// indeed has private key which corresponds to the public key from public key, hence user is expected
	// to sign therefore we should be able to verify signature.
	// 1. User will hash its public key
	// 2. Sign the resulted hash
	// 3. Update signature field with resulted signature
	// 4. Once we receive the information, we will be able to verify user record
	Signature UserSignature `json:"signature,omitempty"`
	Revoked   bool          `json:"revoked,omitempty"`
}

// UserSignature
type UserSignature struct {
	R string `json:"R,omitempty"`
	S string `json:"S,omitempty"`
}

type CustomerType string

const (
	Person  CustomerType = "PERSON"
	Company CustomerType = "COMPANY"
	System  CustomerType = "SYSTEM"
)

func (c CustomerType) String() string {
	return string(c)
}