package nums

import "strconv"

func (obj *CurrencyNumber) DivRaw(b *CurrencyNumber) *CurrencyNumber {
	return obj.Div(NumFromString(b.String(), obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) DivInt(b uint64) *CurrencyNumber {
	return obj.Div(NumFromInt(b, obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) DivFloat(b float64) *CurrencyNumber {
	return obj.Div(NumFromFloat(b, obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) Percent(asset *Asset, percent uint) *CurrencyNumber {
	p := NumFromString(strconv.Itoa(int(percent)), asset.CurrencyType)
	base := NumFromString(strconv.Itoa(100), asset.CurrencyType)

	return obj.Mul(p).Div(base)
}

func (obj *CurrencyNumber) Equals(b *CurrencyNumber) bool {
	return !obj.HasErrors() && !b.HasErrors() && obj.AssetEquals(b) && obj.Num.Cmp(b.Num) == 0
}

func (obj *CurrencyNumber) GreaterEq(b *CurrencyNumber) bool {
	return !obj.HasErrors() && !b.HasErrors() && obj.AssetEquals(b) && obj.Num.Cmp(b.Num) >= 0
}

func (obj *CurrencyNumber) Greater(b *CurrencyNumber) bool {
	return !obj.HasErrors() && !b.HasErrors() && obj.AssetEquals(b) && obj.Num.Cmp(b.Num) > 0
}

func (obj *CurrencyNumber) LessEq(b *CurrencyNumber) bool {
	return !obj.HasErrors() && !b.HasErrors() && obj.AssetEquals(b) && obj.Num.Cmp(b.Num) <= 0
}

func (obj *CurrencyNumber) Less(b *CurrencyNumber) bool {
	return !obj.HasErrors() && !b.HasErrors() && obj.AssetEquals(b) && obj.Num.Cmp(b.Num) < 0
}

func (obj *CurrencyNumber) Inc() *CurrencyNumber {
	return obj.AddSelf(One(obj))
}

func (obj *CurrencyNumber) Dec() *CurrencyNumber {
	return obj.SubSelf(One(obj))
}

func (obj *CurrencyNumber) MulRaw(b *CurrencyNumber) *CurrencyNumber {
	return obj.Mul(NumFromString(b.String(), obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) MulInt(b uint64) *CurrencyNumber {
	return obj.Mul(NumFromInt(b, obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) MulFloat(b float64) *CurrencyNumber {
	return obj.Mul(NumFromFloat(b, obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) MulSelfFloat(b float64) *CurrencyNumber {
	return obj.MulSelf(NumFromFloat(b, obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) MulSelfInt(b uint64) *CurrencyNumber {
	return obj.MulSelf(NumFromInt(b, obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) DivSelfFloat(b float64) *CurrencyNumber {
	return obj.DivSelf(NumFromFloat(b, obj.Asset.CurrencyType))
}

func (obj *CurrencyNumber) DivSelfInt(b uint64) *CurrencyNumber {
	return obj.DivSelf(NumFromInt(b, obj.Asset.CurrencyType))
}
