package actions

import (
	log "github.com/sirupsen/logrus"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
	"strings"
	"errors"
)

type ActionIDsObject struct {
}

type ParameterType int

type Roles string

type SecurityPreference struct {
	ChaincodeName string
	FunctionName  string
	ParameterType ParameterType
	Roles         []Roles
}

var (
	logger = log.WithFields(log.Fields{})
)

func (securityPreference SecurityPreference) GetActionID() int32 {
	actionID := securityPreference.ChaincodeName + securityPreference.FunctionName
	logger.Infof("ActionId source: '%s'", actionID)
	var hash int32
	hash = Hash(actionID)
	logger.Infof("ActionId hash: '%d'", hash)
	return hash
}

func Hash(s string) int32 {
	var h int32
	h = 0
	var ln int32
	var li int32
	ln = int32(len(s))
	li = ln - 1 // li means last index
	var i int32
	for i = 0; i < ln; i++ {
		h += int32(s[i]) * Pow(31, li-i)
	}
	return h
}

func Pow(x int32, y int32) int32 {
	p := x
	if y == 0 {
		return 1
	}
	if y == 1 {
		return x
	}
	// if y < 0, panic?
	var i int32
	for i = 1; i < y; i++ {
		p *= x
	}
	return p
}

const UserParameter ParameterType = 1
const ServiceParameter ParameterType = 1 << 2
const ComplexParameter ParameterType = 1 << 3
const UnsecuredParameter ParameterType = 1 << 4
const BasicRole Roles = "USER"

var SecurityPreferenceArray = []SecurityPreference{

	SecurityPreference{invoke.AccountCC, invoke.AddWalletFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.AddExternalAccountFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.AddAccountFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.ActivateWalletFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.BlockWalletFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.ActivateAccountFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.BlockAccountFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.VerifyUserAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.CheckAndLoadAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetAllUserWalletsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetAccountsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetWalletsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetAccountInfoByAccountIDFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetAccountByCurrencyTypeFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.MarketaccountCC, invoke.AddWalletFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.AddAccountFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.ActivateWalletFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.BlockWalletFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.ActivateAccountFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.BlockAccountFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.VerifyUserAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.CheckAndLoadAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.GetAllUserWalletsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.GetAccountInfoByAccountIDFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.GetAccountByCurrencyTypeFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.AuthCC, invoke.AddAuthInfo, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AuthCC, invoke.UpdateAuthInfo, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.AuthCC, invoke.GetAuthInfo, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.CommissionCC, invoke.AddCommissionFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.CommissionCC, invoke.GetCommissionFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.CommissionCC, invoke.UpdateCommissionFunction, ServiceParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.CryptocurrencyCC, invoke.AddCryptocurrenciesFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.CryptocurrencyCC, invoke.GetCryptocurrenciesByDaemonNameFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.CryptocurrencyCC, invoke.ClearTransferredCryptocurrencyListFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.ExternalofferCC, invoke.AddOfferFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.ExternalofferCC, invoke.GetUserOutgoingRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.ExternalofferCC, invoke.GetRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.ExternalofferCC, invoke.GetHistoryUserOutgoingRequestsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.ExternalofferCC, invoke.UpdateOfferExternalTransactionIdFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.ExternalofferCC, invoke.GetRequestsByStatusAndCurrencyTypeFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.ExternalofferCC, invoke.ChangeOfferStatusByOperationIdFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.ExternalofferCC, invoke.ChangeOfferStatusByExternalTransactionIdFunction, ServiceParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.OfferCC, invoke.AddOfferFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetUserIncomingRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetUserOutgoingRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetHistoryUserIncomingRequestsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetHistoryUserOutgoingRequestsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetOfferFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.ChangeOfferStatusByOperationIdFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.IsOfferApprovedFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.ReplayOfferFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetReplayHistoryOfferFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.TransactionCC, invoke.LockFundsOnAccountFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.AccountCurrencyBalanceFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.TransferFundsFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.GetUsersTransactionFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LoadCryptocurrencyFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LockOutFundsOnAccountFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LockTransferToMarketFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.UnlockFundsOnAccountFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.CheckExistsTransactionByListFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.FindLockAmountByOperationIDAndUserIdFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LoadMarketFoundsFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.DepositMarketFundsFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.TransactionCC, invoke.WithdrawalMarketFundsFunction, ServiceParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.UserCC, invoke.AddRootUserFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.AddOrganizationRootUserFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.AddSystemUserFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.AddServiceUserFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.AddUserFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.GetUserFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.AddPublicKeyFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.RevokeUserPublicKeyFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.VerifyUserPublicKeyFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserCC, invoke.VerifyUserSignatureFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.OrganizationCC, invoke.AddOrganizationFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.OrganizationCC, invoke.GetOrganizationFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.MarketCC, invoke.DepositFundsFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.GetBalanceFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.LockWithdrawalFundsFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.TransferWithdrawalFundsFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.CreateOrderFunction, ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.GetAllOrdersFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.GetUserOrdersFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.CancelOrderFunction, UserParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.MatchOrdersFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.FindOutgoingTransactionsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.MarketCC, invoke.FindIncomingTransactionsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.UserpreferenceCC, invoke.AddPreferenceFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.UserpreferenceCC, invoke.GetIntAccountByExtAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{invoke.SystempreferenceCC, invoke.SetFieldConstraintsFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.SystempreferenceCC, invoke.GetFieldConstraintsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.SystempreferenceCC, invoke.SetEnabledCryptoCurrenciesFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.SystempreferenceCC, invoke.GetEnabledCryptoCurrenciesFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.SystempreferenceCC, invoke.SetCryptoCurrencyConstraintsFunction, ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{invoke.SystempreferenceCC, invoke.GetCryptoCurrencyConstraintsFunction, ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
}

func FindSecurityPreferenceByFunctionName(chaincodeName string, functionName string) (SecurityPreference, error) {
	logging.Get().Infof("chaincodeName: %s", chaincodeName)
	logging.Get().Infof("functionName: %s", functionName)
	chaincodeNameLower := firstLetterToLowerCase(chaincodeName)
	functionNameLower := firstLetterToLowerCase(functionName)

	for _, v := range SecurityPreferenceArray {
		if v.ChaincodeName == chaincodeNameLower && v.FunctionName == functionNameLower {
			logging.Get().Infof("found SecurityPreference: %s", v)
			return v, nil
		}
	}
	return SecurityPreference{}, errors.New("not found")
}

func firstLetterToLowerCase(str string) string {
	firstLetter := strings.ToLower(str[0:1])
	replaced := firstLetter + str[1:]

	return replaced
}
